<?php

/**
 * NukeViet Content Management System
 * @version 4.x
 * @author VINADES.,JSC <contact@vinades.vn>
 * @copyright (C) 2009-2021 VINADES.,JSC. All rights reserved
 * @license GNU/GPL version 2 or any later version
 * @see https://github.com/nukeviet The NukeViet CMS GitHub project
 */

if (!defined('NV_MAINFILE')) {
    exit('Stop!!!');
}

// Japanese
$utf8_lookup_lang = [
    // Japanese hiragana
    'あ' => 'a', 'え' => 'e', 'い' => 'i', 'お' => 'o', 'う' => 'u', 'ば' => 'ba', 'べ' => 'be', 'び' => 'bi', 'ぼ' => 'bo', 'ぶ' => 'bu',
    'し' => 'ci', 'だ' => 'da', 'で' => 'de', 'ぢ' => 'di', 'ど' => 'do', 'づ' => 'du', 'ふぁ' => 'fa', 'ふぇ' => 'fe', 'ふぃ' => 'fi', 'ふぉ' => 'fo',
    'ふ' => 'fu', 'が' => 'ga', 'げ' => 'ge', 'ぎ' => 'gi', 'ご' => 'go', 'ぐ' => 'gu', 'は' => 'ha', 'へ' => 'he', 'ひ' => 'hi', 'ほ' => 'ho',
    'ふ' => 'hu', 'じゃ' => 'ja', 'じぇ' => 'je', 'じ' => 'ji', 'じょ' => 'jo', 'じゅ' => 'ju', 'か' => 'ka', 'け' => 'ke', 'き' => 'ki', 'こ' => 'ko',
    'く' => 'ku', 'ら' => 'la', 'れ' => 'le', 'り' => 'li', 'ろ' => 'lo', 'る' => 'lu', 'ま' => 'ma', 'め' => 'me', 'み' => 'mi', 'も' => 'mo',
    'む' => 'mu', 'な' => 'na', 'ね' => 'ne', 'に' => 'ni', 'の' => 'no', 'ぬ' => 'nu', 'ぱ' => 'pa', 'ぺ' => 'pe', 'ぴ' => 'pi', 'ぽ' => 'po',
    'ぷ' => 'pu', 'ら' => 'ra', 'れ' => 're', 'り' => 'ri', 'ろ' => 'ro', 'る' => 'ru', 'さ' => 'sa', 'せ' => 'se', 'し' => 'si', 'そ' => 'so',
    'す' => 'su', 'た' => 'ta', 'て' => 'te', 'ち' => 'ti', 'と' => 'to', 'つ' => 'tu', 'ヴぁ' => 'va', 'ヴぇ' => 've', 'ヴぃ' => 'vi', 'ヴぉ' => 'vo',
    'ヴ' => 'vu', 'わ' => 'wa', 'うぇ' => 'we', 'うぃ' => 'wi', 'を' => 'wo', 'や' => 'ya', 'いぇ' => 'ye', 'い' => 'yi', 'よ' => 'yo', 'ゆ' => 'yu',
    'ざ' => 'za', 'ぜ' => 'ze', 'じ' => 'zi', 'ぞ' => 'zo', 'ず' => 'zu', 'びゃ' => 'bya', 'びぇ' => 'bye', 'びぃ' => 'byi', 'びょ' => 'byo', 'びゅ' => 'byu',
    'ちゃ' => 'cha', 'ちぇ' => 'che', 'ち' => 'chi', 'ちょ' => 'cho', 'ちゅ' => 'chu', 'ちゃ' => 'cya', 'ちぇ' => 'cye', 'ちぃ' => 'cyi', 'ちょ' => 'cyo',
    'ちゅ' => 'cyu', 'でゃ' => 'dha', 'でぇ' => 'dhe', 'でぃ' => 'dhi', 'でょ' => 'dho', 'でゅ' => 'dhu', 'どぁ' => 'dwa', 'どぇ' => 'dwe', 'どぃ' => 'dwi',
    'どぉ' => 'dwo', 'どぅ' => 'dwu', 'ぢゃ' => 'dya', 'ぢぇ' => 'dye', 'ぢぃ' => 'dyi', 'ぢょ' => 'dyo', 'ぢゅ' => 'dyu', 'ぢ' => 'dzi', 'ふぁ' => 'fwa',
    'ふぇ' => 'fwe', 'ふぃ' => 'fwi', 'ふぉ' => 'fwo', 'ふぅ' => 'fwu', 'ふゃ' => 'fya', 'ふぇ' => 'fye', 'ふぃ' => 'fyi', 'ふょ' => 'fyo', 'ふゅ' => 'fyu',
    'ぎゃ' => 'gya', 'ぎぇ' => 'gye', 'ぎぃ' => 'gyi', 'ぎょ' => 'gyo', 'ぎゅ' => 'gyu', 'ひゃ' => 'hya', 'ひぇ' => 'hye', 'ひぃ' => 'hyi', 'ひょ' => 'hyo',
    'ひゅ' => 'hyu', 'じゃ' => 'jya', 'じぇ' => 'jye', 'じぃ' => 'jyi', 'じょ' => 'jyo', 'じゅ' => 'jyu', 'きゃ' => 'kya', 'きぇ' => 'kye', 'きぃ' => 'kyi',
    'きょ' => 'kyo', 'きゅ' => 'kyu', 'りゃ' => 'lya', 'りぇ' => 'lye', 'りぃ' => 'lyi', 'りょ' => 'lyo', 'りゅ' => 'lyu', 'みゃ' => 'mya', 'みぇ' => 'mye',
    'みぃ' => 'myi', 'みょ' => 'myo', 'みゅ' => 'myu', 'ん' => 'n', 'にゃ' => 'nya', 'にぇ' => 'nye', 'にぃ' => 'nyi', 'にょ' => 'nyo', 'にゅ' => 'nyu',
    'ぴゃ' => 'pya', 'ぴぇ' => 'pye', 'ぴぃ' => 'pyi', 'ぴょ' => 'pyo', 'ぴゅ' => 'pyu', 'りゃ' => 'rya', 'りぇ' => 'rye', 'りぃ' => 'ryi', 'りょ' => 'ryo',
    'りゅ' => 'ryu', 'しゃ' => 'sha', 'しぇ' => 'she', 'し' => 'shi', 'しょ' => 'sho', 'しゅ' => 'shu', 'すぁ' => 'swa', 'すぇ' => 'swe', 'すぃ' => 'swi',
    'すぉ' => 'swo', 'すぅ' => 'swu', 'しゃ' => 'sya', 'しぇ' => 'sye', 'しぃ' => 'syi', 'しょ' => 'syo', 'しゅ' => 'syu', 'てゃ' => 'tha', 'てぇ' => 'the',
    'てぃ' => 'thi', 'てょ' => 'tho', 'てゅ' => 'thu', 'つゃ' => 'tsa', 'つぇ' => 'tse', 'つぃ' => 'tsi', 'つょ' => 'tso', 'つ' => 'tsu', 'とぁ' => 'twa',
    'とぇ' => 'twe', 'とぃ' => 'twi', 'とぉ' => 'two', 'とぅ' => 'twu', 'ちゃ' => 'tya', 'ちぇ' => 'tye', 'ちぃ' => 'tyi', 'ちょ' => 'tyo', 'ちゅ' => 'tyu',
    'ヴゃ' => 'vya', 'ヴぇ' => 'vye', 'ヴぃ' => 'vyi', 'ヴょ' => 'vyo', 'ヴゅ' => 'vyu', 'うぁ' => 'wha', 'うぇ' => 'whe', 'うぃ' => 'whi', 'うぉ' => 'who',
    'うぅ' => 'whu', 'ゑ' => 'wye', 'ゐ' => 'wyi', 'じゃ' => 'zha', 'じぇ' => 'zhe', 'じぃ' => 'zhi', 'じょ' => 'zho', 'じゅ' => 'zhu', 'じゃ' => 'zya',
    'じぇ' => 'zye', 'じぃ' => 'zyi', 'じょ' => 'zyo', 'じゅ' => 'zyu',
    // Japanese katakana
    'ア' => 'a', 'エ' => 'e', 'イ' => 'i', 'オ' => 'o', 'ウ' => 'u', 'バ' => 'ba', 'ベ' => 'be', 'ビ' => 'bi', 'ボ' => 'bo', 'ブ' => 'bu',
    'シ' => 'ci', 'ダ' => 'da', 'デ' => 'de', 'ヂ' => 'di', 'ド' => 'do', 'ヅ' => 'du', 'ファ' => 'fa', 'フェ' => 'fe', 'フィ' => 'fi', 'フォ' => 'fo',
    'フ' => 'fu', 'ガ' => 'ga', 'ゲ' => 'ge', 'ギ' => 'gi', 'ゴ' => 'go', 'グ' => 'gu', 'ハ' => 'ha', 'ヘ' => 'he', 'ヒ' => 'hi', 'ホ' => 'ho',
    'フ' => 'hu', 'ジャ' => 'ja', 'ジェ' => 'je', 'ジ' => 'ji', 'ジョ' => 'jo', 'ジュ' => 'ju', 'カ' => 'ka', 'ケ' => 'ke', 'キ' => 'ki', 'コ' => 'ko',
    'ク' => 'ku', 'ラ' => 'la', 'レ' => 'le', 'リ' => 'li', 'ロ' => 'lo', 'ル' => 'lu', 'マ' => 'ma', 'メ' => 'me', 'ミ' => 'mi', 'モ' => 'mo',
    'ム' => 'mu', 'ナ' => 'na', 'ネ' => 'ne', 'ニ' => 'ni', 'ノ' => 'no', 'ヌ' => 'nu', 'パ' => 'pa', 'ペ' => 'pe', 'ピ' => 'pi', 'ポ' => 'po',
    'プ' => 'pu', 'ラ' => 'ra', 'レ' => 're', 'リ' => 'ri', 'ロ' => 'ro', 'ル' => 'ru', 'サ' => 'sa', 'セ' => 'se', 'シ' => 'si', 'ソ' => 'so',
    'ス' => 'su', 'タ' => 'ta', 'テ' => 'te', 'チ' => 'ti', 'ト' => 'to', 'ツ' => 'tu', 'ヴァ' => 'va', 'ヴェ' => 've', 'ヴィ' => 'vi', 'ヴォ' => 'vo',
    'ヴ' => 'vu', 'ワ' => 'wa', 'ウェ' => 'we', 'ウィ' => 'wi', 'ヲ' => 'wo', 'ヤ' => 'ya', 'イェ' => 'ye', 'イ' => 'yi', 'ヨ' => 'yo', 'ユ' => 'yu',
    'ザ' => 'za', 'ゼ' => 'ze', 'ジ' => 'zi', 'ゾ' => 'zo', 'ズ' => 'zu', 'ビャ' => 'bya', 'ビェ' => 'bye', 'ビィ' => 'byi', 'ビョ' => 'byo',
    'ビュ' => 'byu', 'チャ' => 'cha', 'チェ' => 'che', 'チ' => 'chi', 'チョ' => 'cho', 'チュ' => 'chu', 'チャ' => 'cya', 'チェ' => 'cye', 'チィ' => 'cyi',
    'チョ' => 'cyo', 'チュ' => 'cyu', 'デャ' => 'dha', 'デェ' => 'dhe', 'ディ' => 'dhi', 'デョ' => 'dho', 'デュ' => 'dhu', 'ドァ' => 'dwa', 'ドェ' => 'dwe',
    'ドィ' => 'dwi', 'ドォ' => 'dwo', 'ドゥ' => 'dwu', 'ヂャ' => 'dya', 'ヂェ' => 'dye', 'ヂィ' => 'dyi', 'ヂョ' => 'dyo', 'ヂュ' => 'dyu', 'ヂ' => 'dzi',
    'ファ' => 'fwa', 'フェ' => 'fwe', 'フィ' => 'fwi', 'フォ' => 'fwo', 'フゥ' => 'fwu', 'フャ' => 'fya', 'フェ' => 'fye', 'フィ' => 'fyi', 'フョ' => 'fyo',
    'フュ' => 'fyu', 'ギャ' => 'gya', 'ギェ' => 'gye', 'ギィ' => 'gyi', 'ギョ' => 'gyo', 'ギュ' => 'gyu', 'ヒャ' => 'hya', 'ヒェ' => 'hye', 'ヒィ' => 'hyi',
    'ヒョ' => 'hyo', 'ヒュ' => 'hyu', 'ジャ' => 'jya', 'ジェ' => 'jye', 'ジィ' => 'jyi', 'ジョ' => 'jyo', 'ジュ' => 'jyu', 'キャ' => 'kya', 'キェ' => 'kye',
    'キィ' => 'kyi', 'キョ' => 'kyo', 'キュ' => 'kyu', 'リャ' => 'lya', 'リェ' => 'lye', 'リィ' => 'lyi', 'リョ' => 'lyo', 'リュ' => 'lyu', 'ミャ' => 'mya',
    'ミェ' => 'mye', 'ミィ' => 'myi', 'ミョ' => 'myo', 'ミュ' => 'myu', 'ン' => 'n', 'ニャ' => 'nya', 'ニェ' => 'nye', 'ニィ' => 'nyi', 'ニョ' => 'nyo',
    'ニュ' => 'nyu', 'ピャ' => 'pya', 'ピェ' => 'pye', 'ピィ' => 'pyi', 'ピョ' => 'pyo', 'ピュ' => 'pyu', 'リャ' => 'rya', 'リェ' => 'rye', 'リィ' => 'ryi',
    'リョ' => 'ryo', 'リュ' => 'ryu', 'シャ' => 'sha', 'シェ' => 'she', 'シ' => 'shi', 'ショ' => 'sho', 'シュ' => 'shu', 'スァ' => 'swa', 'スェ' => 'swe',
    'スィ' => 'swi', 'スォ' => 'swo', 'スゥ' => 'swu', 'シャ' => 'sya', 'シェ' => 'sye', 'シィ' => 'syi', 'ショ' => 'syo', 'シュ' => 'syu', 'テャ' => 'tha',
    'テェ' => 'the', 'ティ' => 'thi', 'テョ' => 'tho', 'テュ' => 'thu', 'ツャ' => 'tsa', 'ツェ' => 'tse', 'ツィ' => 'tsi', 'ツョ' => 'tso', 'ツ' => 'tsu',
    'トァ' => 'twa', 'トェ' => 'twe', 'トィ' => 'twi', 'トォ' => 'two', 'トゥ' => 'twu', 'チャ' => 'tya', 'チェ' => 'tye', 'チィ' => 'tyi', 'チョ' => 'tyo',
    'チュ' => 'tyu', 'ヴャ' => 'vya', 'ヴェ' => 'vye', 'ヴィ' => 'vyi', 'ヴョ' => 'vyo', 'ヴュ' => 'vyu', 'ウァ' => 'wha', 'ウェ' => 'whe', 'ウィ' => 'whi',
    'ウォ' => 'who', 'ウゥ' => 'whu', 'ヱ' => 'wye', 'ヰ' => 'wyi', 'ジャ' => 'zha', 'ジェ' => 'zhe', 'ジィ' => 'zhi', 'ジョ' => 'zho', 'ジュ' => 'zhu',
    'ジャ' => 'zya', 'ジェ' => 'zye', 'ジィ' => 'zyi', 'ジョ' => 'zyo', 'ジュ' => 'zyu'
];
